<?php
/**
 * NewCustomerRequest.php
 * Copyright (c) 2019 thegrumpydictator@gmail.com
 *
 * This file is part of Firefly III (https://github.com/firefly-iii).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
declare(strict_types=1);

namespace FireflyIII\Services\Spectre\Request;

use FireflyIII\Services\Spectre\Object\Customer;
use Log;

/**
 * Class NewCustomerRequest
 * @codeCoverageIgnore
 */
class NewCustomerRequest extends SpectreRequest
{
    /** @var Customer */
    protected $customer;

    /**
     * @throws \FireflyIII\Exceptions\FireflyException
     */
    public function call(): void
    {
        $data = [
            'data' => [
                'identifier' => 'default_ff3_customer',
            ],
        ];
        $uri  = '/api/v4/customers/';
        Log::debug(sprintf('Going to call %s with info:', $uri), $data);
        $response = $this->sendSignedSpectrePost($uri, $data);
        // create customer:
        $this->customer = new Customer($response['data']);
    }

    /**
     * @return Customer
     */
    public function getCustomer(): Customer
    {
        return $this->customer;
    }
}
